import { Subject } from 'rxjs/Rx';

export class ObservableArray<T> extends Subject<T[]> {
    private _array: T[];
    constructor(array?: T[]) {
        super();
        this._array = array || [];
    }

    set array(value: T[]) {
        this._array = value;
        this.next(this._array);
    }

    get array(): T[] {
        return this._array;
    }

    push(item: T) {
        this._array.push(item);
        this.next(this._array);
    }

    remove(valueOrPredicate: any): T[] {
        let predicate = typeof valueOrPredicate === 'function'
                        ? valueOrPredicate
                        : function (value) { return value === valueOrPredicate; };
        let removedValues = [];
        for (let i = 0; i < this._array.length; i++) {
            let value = this._array[i];
            if (predicate(value)) {
                removedValues.push(value);
                this._array.splice(i, 1);
                i--;
            }
        }

        this.next(this._array);

        return removedValues;
    }
}
